﻿init python:
    # preferences.gl_framerate = 60
    def aabb_collision(box_1, box_2):
        if box_1.x < box_2.x + box_2.width:
            return False
        if box_1.x + box_1.width > box_2.x:
            return False
        if box_1.y < box_2.y + box_2.height:
            return False
        if box_1.y + box_1.height > box_2.y:
            return False
        return True

    def load_map(tilemap):
        map = []
        f = open(renpy.loader.transfn(tilemap),"r")
        for line in f:
            line = line.strip()
            divided = line.split(",")
            listed = [int(x) for x in divided]
            map.append(listed)
        f.close()
        return map
    def load_tiles(img, size, y = 8, x = 8):
        tiles = []
        for i in range(x):
            for ii in range(y):
                tiles.append(Crop((ii*size,i*size,size,size), img))
        return tiles

    def construct_map(map, size):
        result = []
        for y,j in enumerate(map):
            for x,i in enumerate(j):
                result.append(tile(i, x*size, y*size, size))
        return result

    def make_collision_map(map, floors):
        result = []
        for y in map:
            row = []
            for x in y:
                if x in floors:
                    row.append(0)
                else:
                    row.append(1)
            result.append(row)
        return result

    def find_chairs(map, chairs, table):
        chair_list = []
        for y,j in enumerate(map):
            for x,i in enumerate(j):
                if i in chairs:
                    chair = {
                        "x":x,
                        "y":y,
                        "table_x":0,
                        "table_y":0,
                        "xzoom":0,
                    }
                    if map[y][x-1] == table:
                        chair["table_x"] = x - 1
                        chair["table_y"] = y
                        chair["xzoom"] = -1
                    elif map[y][x+1] == table:
                        chair["table_x"] = x + 1
                        chair["table_y"] = y
                        chair["xzoom"] = -1

                    chair_list.append(chair)
                    


    class animation_class:
        def __init__(self, frames):
            self.frames = frames
            self.current_delta = 0
            self.current_index = 0
        def get_frame(self, delta):
            self.current_delta += delta
            current_frame = self.frames[self.current_index]
            if self.current_delta > current_frame[1]:
                self.current_delta = 0
                self.current_index += 1
                if self.current_index > len(self.frames)-1:
                    self.current_index = 0
            return self.frames[self.current_index][0]

    class animation_handler:
        def __init__(self, animations):
            self.animations = animations
            self.current_animation = None

        def get_frame(self, delta, animation):
            if type(self.animations[animation]) is str:
                return self.animations[animation]
            if not self.current_animation == animation:
                self.current_animation = animation
                return self.animations[animation].get_frame(delta)
            return self.animations[animation].get_frame(delta)
            
    class tile:
        def __init__(self, image, x, y, size):
            self.image = image
            self.x = x
            self.y = y
            self.max_x = x + size
            self.max_y = y + size

    class actor_obj(renpy.Displayable):
        def __init__(self, animations, parent):
            super(renpy.Displayable,self).__init__()
            self.animations = animations
            self.x = 128
            self.y = 128
            self.speed = 90
            self.facing = 0
            self.frame = 0
            self.animation = "down_idle"
            self.last_time = 0
            self.delta_time = 0
            self.parent = parent

        def move(self, move):
            self.animation = move
        def stop(self):
            if self.animation in ["up_idle", "down_idle", "left_idle", "right_idle"]:
                return
            self.animation = self.animation + "_idle"
        def render(self,width,height,st,at):
            if self.last_time == 0:
                self.last_time = time.perf_counter()
            current_time = time.perf_counter()
            delta_time =  round(current_time - self.last_time, 4)
            self.delta_time = delta_time
            if self.animation in ["up_idle", "down_idle", "left_idle", "right_idle"]:
                pass
            else:
                x = int(self.x + 16)
                y = int(self.y + 22)
                if self.animation == "up":
                    left_hand_x, left_hand_y = x - 8 , y - 10
                    right_hand_x, right_hand_y = x + 8 , y - 10
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.y -= self.speed*delta_time
                elif self.animation == "down":
                    left_hand_x, left_hand_y = x + 8 , y + 10
                    right_hand_x, right_hand_y = x - 8 , y + 10
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.y += self.speed*delta_time
                elif self.animation == "right":
                    left_hand_x, left_hand_y = x + 10 , y + 8
                    right_hand_x, right_hand_y = x + 10 , y - 8
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.x += self.speed*delta_time
                elif self.animation == "left":
                    left_hand_x, left_hand_y = x - 10 , y - 8
                    right_hand_x, right_hand_y = x - 10 , y + 8
                    if self.parent.collision_map[left_hand_y//32][left_hand_x//32] or self.parent.collision_map[right_hand_y//32][right_hand_x//32]:
                        self.stop()
                    else:
                        self.x -= self.speed*delta_time
            

            image = self.animations.get_frame(delta_time * 1000, self.animation)
            self.last_time = current_time
            t = Transform(image, xanchor=0.5, yanchor=0.5)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

    class topdown_handler:
        def __init__(self, map, tileset, size):
            self.stat = "start"
            self.player = actor_obj(main_animation_frames, self)
            self.current_move = None
            self.map = load_map(map)
            self.collision_map = make_collision_map(self.map, [18])
            self.tileset = load_tiles(tileset, size, 8, 4)
            self.size = size
            self.chairs = find_chairs(map, [20,22], 21)

        def tick(self):
            pass
        def start(self):
            pass
        def end(self):
            pass
        def spawn_food(self):
            pass
        def spawn_client(self):
            pass
        def move(self, move):
            if self.current_move == move:
                self.current_move = None
                self.player.stop()
                return
            self.current_move = move
            self.player.move(move)

screen topdown_screen():
    default g = topdown_handler("topdown/map_1.csv", "topdown/atlas.png", 32)
    style_prefix "topdown"
    fixed xysize 640,352:
        at zoom(3, True)
        vbox spacing 0 align (0.5, 0.5):
            for yp,y in enumerate(g.map):
                hbox spacing 0:
                    for xp,x in enumerate(y):
                        frame xysize(g.size,g.size):
                            if x in [-1]:
                                background None
                            else:
                                background g.tileset[x]
                            # text str(g.collision_map[yp][xp]) size 10 align .5,.5
        add g.player

    text str(g.player.delta_time) align .1,.0

    fixed xysize 100,100 align 0.0,1.0 offset 150,-150:
        button xysize 100,100:
            yoffset -100 background "#fff5"
            text "w" align .5,.5
            action Function(g.move, "up") keysym ["K_UP", "w"]
        button xysize 100,100:
            xoffset -100 background "#fff5"
            text "a" align .5,.5
            action Function(g.move, "left") keysym ["K_LEFT", "a"]
        button xysize 100,100:
            yoffset 100 background "#fff5"
            text "s" align .5,.5
            action Function(g.move, "down") keysym ["K_DOWN", "s"]
        button xysize 100,100:
            xoffset 100 background "#fff5"
            text "d" align .5,.5
            action Function(g.move, "right") keysym ["K_RIGHT", "d"]


init: #styles
    style topdown_text:
        align (.5,.5)
    style topdown_frame:
        align (.5,.5)
    style topdown_button:
        align (.5,.5)

init: # main character
    image main_down_1:
        Crop((0, 0, 32, 32), "topdown/main.png")
    image main_down_2:
        Crop((32, 0, 32, 32), "topdown/main.png")
    image main_down_3:
        Crop((64, 0, 32, 32), "topdown/main.png")

    image main_left_1:
        Crop((0, 32, 32, 32), "topdown/main.png")
    image main_left_2:
        Crop((32, 32, 32, 32), "topdown/main.png")
    image main_left_3:
        Crop((64, 32, 32, 32), "topdown/main.png")

    image main_up_1:
        Crop((0, 64, 32, 32), "topdown/main.png")
    image main_up_2:
        Crop((32, 64, 32, 32), "topdown/main.png")
    image main_up_3:
        Crop((64, 64, 32, 32), "topdown/main.png")

    image main_right_1:
        Crop((0, 96, 32, 32), "topdown/main.png")
    image main_right_2:
        Crop((32, 96, 32, 32), "topdown/main.png")
    image main_right_3:
        Crop((64, 96, 32, 32), "topdown/main.png")

    default main_animation_up = animation_class(
            [
                ["main_up_1", 150],
                ["main_up_2", 150],
                ["main_up_3", 150],
                ["main_up_2", 150],
            ]
        )
    default main_animation_left = animation_class(
            [
                ["main_left_1", 150],
                ["main_left_2", 150],
                ["main_left_3", 150],
                ["main_left_2", 150],
            ]
        )
    default main_animation_down = animation_class(
            [
                ["main_down_1", 150],
                ["main_down_2", 150],
                ["main_down_3", 150],
                ["main_down_2", 150],
            ]
        )
    default main_animation_right = animation_class(
            [
                ["main_right_1", 150],
                ["main_right_2", 150],
                ["main_right_3", 150],
                ["main_right_2", 150],
            ]
        )
    
    default main_animation_frames = animation_handler(
            {
                "up": main_animation_up,
                "left": main_animation_left,
                "down": main_animation_down,
                "right": main_animation_right,

                "up_idle": "main_up_2",
                "left_idle": "main_left_2",
                "down_idle": "main_down_2",
                "right_idle": "main_right_2",
            }
        )

init: # food
    image topdown_table:
        Crop((0, 0, 32, 32), "topdown/atlas.png")


label topdown_example:
    call screen topdown_screen




